import { __assign } from 'tslib';
import sync, { getFrameData } from 'framesync';
import { copyAxisBox } from '../../../utils/geometry/index.js';
import { compareByDepth } from '../../utils/compare-by-depth.js';

function isProjecting(visualElement) {
    var isEnabled = visualElement.projection.isEnabled;
    return isEnabled || visualElement.shouldResetTransform();
}
function collectProjectingAncestors(visualElement, ancestors) {
    if (ancestors === void 0) { ancestors = []; }
    var parent = visualElement.parent;
    if (parent)
        collectProjectingAncestors(parent, ancestors);
    if (isProjecting(visualElement))
        ancestors.push(visualElement);
    return ancestors;
}
function collectProjectingChildren(visualElement) {
    var children = [];
    var addChild = function (child) {
        if (isProjecting(child))
            children.push(child);
        child.children.forEach(addChild);
    };
    visualElement.children.forEach(addChild);
    return children.sort(compareByDepth);
}
/**
 * Update the layoutState by measuring the DOM layout. This
 * should be called after resetting any layout-affecting transforms.
 */
function updateLayoutMeasurement(visualElement, rebase) {
    if (rebase === void 0) { rebase = true; }
    if (visualElement.shouldResetTransform())
        return;
    var layoutState = visualElement.getLayoutState();
    visualElement.notifyBeforeLayoutMeasure(layoutState.layout);
    layoutState.isHydrated = true;
    layoutState.layout = visualElement.measureViewportBox();
    layoutState.layoutCorrected = copyAxisBox(layoutState.layout);
    var snapshot = visualElement.snapshot;
    visualElement.notifyLayoutMeasure(layoutState.layout, snapshot ? snapshot.viewportBox : layoutState.layout);
    rebase && sync.update(function () { return visualElement.rebaseProjectionTarget(); });
}
/**
 * Record the viewport box as it was before an expected mutation/re-render
 */
function snapshotViewportBox(visualElement, rebase) {
    if (rebase === void 0) { rebase = true; }
    if (visualElement.shouldResetTransform())
        return;
    visualElement.snapshot = {
        taken: getFrameData().timestamp,
        transform: __assign({}, visualElement.getLatestValues()),
        viewportBox: visualElement.measureViewportBox(false),
    };
    /**
     * Update targetBox to match the snapshot. This is just to ensure
     * that targetBox is affected by scroll in the same way as the measured box
     */
    rebase &&
        visualElement.rebaseProjectionTarget(false, visualElement.snapshot.viewportBox);
}

export { collectProjectingAncestors, collectProjectingChildren, snapshotViewportBox, updateLayoutMeasurement };
